-------------------------------------------------------------------------------
-- Title      : constants.vhd
-- Project    : DIOT Zynq Ultrascale-based System Board
-------------------------------------------------------------------------------
-- File       : constants.vhd
-- Company    : CERN
-- Platform   : FPGA-generics
-- Description: This is a module used to provide constant values to signals in
--              other IP's. It also contains behavioral logic for I2C and 
--              emio gpios.
-------------------------------------------------------------------------------
-- Copyright (c) 2019 CERN
--
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 0.51 (the "License") (which enables you, at your option,
-- to treat this file as licensed under the Apache License 2.0); you may not
-- use this file except in compliance with the License. You may obtain a copy
-- of the License at http://solderpad.org/licenses/SHL-0.51.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
-------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity constants is
  generic (
        g_phyaddr       : integer := 9;
        g_config_vector : integer := 0;
        g_config_valid  : integer := 0;
        g_signal_detect : integer := 1);
  Port (
        pl_clk_i        : in std_logic; 
        -- Vector indicating status. We need bits 0 and 1 of the vector
        status_vector   : in std_logic_vector(15 downto 0);
        -- Management clock, (<= 2.5MHz) 
        mdc_i           : in std_logic;
  		mdc_clk_led 	: out std_logic;
        -- Reset of the module, coming from PS
        pl_resetn_i      : in std_logic;
        reset_o 	    : out std_logic;
        -- Autoneg
        an_config_o      : out std_logic;
        an_config_vec_o  : out std_logic_vector(15 downto 0);
        
        -- EMIO GPIOs
        p_pres_i        : in  std_logic_vector(1 downto 0);
        pwr_cycle_req_o : out std_logic;
        ps_emio_o       : out std_logic_vector(4 downto 0);
        ps_emio_i       : in  std_logic_vector(4 downto 0);
        
        -- I2C busses
        ps_i2c1_sda_i   : in  std_logic;
        ps_i2c1_sda_t_i : in  std_logic;
        ps_i2c1_sda_o   : out std_logic;
        ps_i2c1_scl_i   : in  std_logic;
        ps_i2c1_scl_t_i : in  std_logic;
        ps_i2c1_scl_o   : out std_logic;
        
        --wrflash_sda_i   : in  std_logic;
        --wrflash_sda_o   : out std_logic;
        --wrflash_sda_t_o : out std_logic;
        --wrflash_scl_i   : in  std_logic;
        --wrflash_scl_o   : out std_logic;
        --wrflash_scl_t_o : out std_logic;
        
        board_sda_i   : in  std_logic;
        board_sda_o   : out std_logic;
        board_sda_t_o : out std_logic;
        board_scl_i   : in  std_logic;
        board_scl_o   : out std_logic;
        board_scl_t_o : out std_logic;

        -- Constants module
        -- Addr of the PHY device(slave). A constant number
	    phyaddr              : out std_logic_vector(4 downto 0);
        configuration_vector : out std_logic_vector(4 downto 0);
        configuration_valid  : out std_logic_vector(0 downto 0);
        --'1' (if not connected to an optical module), otherwise, default is '0' 
        signal_detect        : out std_logic_vector(0 downto 0);  
        
	    -- Slices module
        link_status_led : out std_logic;
        link_sync_led   : out std_logic);
end constants;

architecture Behavioral of constants is

       -- Because we want to check if mdc_i<=2.5MHz, cnt must be up to 4096
       signal s_cnt 	  : unsigned(11 downto 0);
	   signal s_mdc_led_o : std_logic;
	   
	   type t_pwr_state is (IDLE, RST);
       signal pwr_state : t_pwr_state;
       signal ps_emio2_d0 : std_logic;
       signal ps_emio2_p  : std_logic;
	   
begin

    an_config_o <= '0';
    an_config_vec_o <= x"D801";

    phyaddr              <= std_logic_vector(to_unsigned(g_phyaddr, phyaddr'length));
    configuration_vector <= std_logic_vector(to_unsigned(g_config_vector, configuration_vector'length));
    configuration_valid  <= std_logic_vector(to_unsigned(g_config_valid, configuration_valid'length));
    signal_detect        <= std_logic_vector(to_unsigned(g_signal_detect, signal_detect'length));
    
    ps_emio_o <= p_pres_i(1 downto 0) & "000";
    
    reset_o <= '1' when (pl_resetn_i = '0') else
               '0';   
         
    link_status_led <= status_vector(0);
    link_sync_led   <= status_vector(1);
    
    calc_mdc:process(mdc_i, pl_resetn_i)
    begin
    
	    if (pl_resetn_i = '0') then
		    s_cnt <= (others=>'0');
		    s_mdc_led_o <= '0';
		    
	    elsif (rising_edge(mdc_i)) then
	    
		    if (s_cnt = 4096-2) then
			    s_cnt 	<= (others=>'0');
		    	s_mdc_led_o <= '1';
	    	else
		    	    s_mdc_led_o <= '0';
			    s_cnt <= s_cnt + 1;
	    	end if;
        end if;
            
    end process;

    mdc_clk_led <= s_mdc_led_o;
    
    ps_emio2_p <= '1' when (ps_emio2_d0 = '0' and ps_emio_i(2) = '1') else
                  '0';
    process(pl_clk_i)
    begin
      if rising_edge(pl_clk_i) then
        ps_emio2_d0 <= ps_emio_i(2);
        if pl_resetn_i = '0' or (ps_emio2_p = '1' and ps_emio_i(1) = '1') then
          pwr_state <= IDLE;
          pwr_cycle_req_o <= '0';
        else
  
          if pwr_state = IDLE then
            pwr_cycle_req_o <= '0';
            if ps_emio2_p = '1' and ps_emio_i(1) = '0' then
              pwr_state <= RST;
            end if;
  
          elsif pwr_state = RST then
            pwr_cycle_req_o <= '1';
          end if;
        end if;
      end if;
    end process;
    
    -- I2C: access board i2c and WR/MAC eeprom from a single i2c master
    --ps_i2c1_sda_o <= wrflash_sda_i and board_sda_i;
    ps_i2c1_sda_o <= board_sda_i;
    --wrflash_sda_o <= ps_i2c1_sda_i;
    --wrflash_sda_t_o <= ps_i2c1_sda_t_i;
    board_sda_o   <= ps_i2c1_sda_i;
    board_sda_t_o <= ps_i2c1_sda_t_i;
    
    --ps_i2c1_scl_o <= wrflash_scl_i and board_scl_i;
    ps_i2c1_scl_o <= board_scl_i;
    --wrflash_scl_o <= ps_i2c1_scl_i;
    --wrflash_scl_t_o <= ps_i2c1_scl_t_i;
    board_scl_o   <= ps_i2c1_scl_i;
    board_scl_t_o <= ps_i2c1_scl_t_i;
    

end Behavioral;