-------------------------------------------------------------------------------
-- Title      : diot system board golden design
-- Project    : DIOT Zynq Ultrascale-based System Board
-------------------------------------------------------------------------------
-- File       : diot_sb_golden.vhd
-- Company    : CERN
-- Description: Top level of the diot system board golden design. Contains all
--              the Xilinx and custom IP cores that are used.
-------------------------------------------------------------------------------
-- Copyright (c) 2020 CERN
--
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 0.51 (the "License") (which enables you, at your option,
-- to treat this file as licensed under the Apache License 2.0); you may not
-- use this file except in compliance with the License. You may obtain a copy
-- of the License at http://solderpad.org/licenses/SHL-0.51.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.VComponents.all;

-- Entity which has the MAIN signals
entity diot_sb_golden is
port (
    -- reference clock        
    gtrefclk_in_clk_n : in STD_LOGIC;
    gtrefclk_in_clk_p : in STD_LOGIC;
    M_SCL_0           : inout STD_LOGIC_VECTOR ( 0 to 0 );
    M_SDA_0           : inout STD_LOGIC_VECTOR ( 0 to 0 );
    emio_i2c_scl      : inout STD_LOGIC_VECTOR ( 0 to 0 );
    emio_i2c_sda      : inout STD_LOGIC_VECTOR ( 0 to 0 );
    link_status_led   : out STD_LOGIC_VECTOR ( 0 to 0 );
    link_sync_led     : out STD_LOGIC_VECTOR ( 0 to 0 );
    mdc_clk_led       : out STD_LOGIC_VECTOR ( 0 to 0 );
    p_pres_i_0        : in STD_LOGIC_VECTOR ( 1 downto 0 );
    pl_reset_led      : out STD_LOGIC;
    pwr_cycle_req_o_0 : out STD_LOGIC;
    sfp_rxn           : in STD_LOGIC;
    sfp_rxp           : in STD_LOGIC;
    sfp_txn           : out STD_LOGIC;
    sfp_txp           : out STD_LOGIC;
    wrflash_scl       : inout STD_LOGIC_VECTOR ( 0 to 0 );
    wrflash_sda       : inout STD_LOGIC_VECTOR ( 0 to 0 )
);
end entity diot_sb_golden;

architecture arch of diot_sb_golden is

-----------------------------------------------------------------------
-- Components
-----------------------------------------------------------------------

	-- we name it like this because this name
	-- was given by default
	component design_1_zynq_ultra_ps_e_0_0 is
	port (
		maxihpm0_fpd_aclk : IN STD_LOGIC;
		maxigp0_awid : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		maxigp0_awaddr : OUT STD_LOGIC_VECTOR(39 DOWNTO 0);
		maxigp0_awlen : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		maxigp0_awsize : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
		maxigp0_awburst : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		maxigp0_awlock : OUT STD_LOGIC;
		maxigp0_awcache : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		maxigp0_awprot : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
		maxigp0_awvalid : OUT STD_LOGIC;
		maxigp0_awuser : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		maxigp0_awready : IN STD_LOGIC;
		maxigp0_wdata : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
		maxigp0_wstrb : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		maxigp0_wlast : OUT STD_LOGIC;
		maxigp0_wvalid : OUT STD_LOGIC;
		maxigp0_wready : IN STD_LOGIC;
		maxigp0_bid : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		maxigp0_bresp : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		maxigp0_bvalid : IN STD_LOGIC;
		maxigp0_bready : OUT STD_LOGIC;
		maxigp0_arid : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		maxigp0_araddr : OUT STD_LOGIC_VECTOR(39 DOWNTO 0);
		maxigp0_arlen : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		maxigp0_arsize : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
		maxigp0_arburst : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		maxigp0_arlock : OUT STD_LOGIC;
		maxigp0_arcache : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		maxigp0_arprot : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
		maxigp0_arvalid : OUT STD_LOGIC;
		maxigp0_aruser : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		maxigp0_arready : IN STD_LOGIC;
		maxigp0_rid : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		maxigp0_rdata : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
		maxigp0_rresp : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		maxigp0_rlast : IN STD_LOGIC;
		maxigp0_rvalid : IN STD_LOGIC;
		maxigp0_rready : OUT STD_LOGIC;
		maxigp0_awqos : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		maxigp0_arqos : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		emio_enet0_gmii_rx_clk : IN STD_LOGIC;
		emio_enet0_speed_mode : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
		emio_enet0_gmii_crs : IN STD_LOGIC;
		emio_enet0_gmii_col : IN STD_LOGIC;
		emio_enet0_gmii_rxd : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		emio_enet0_gmii_rx_er : IN STD_LOGIC;
		emio_enet0_gmii_rx_dv : IN STD_LOGIC;
		emio_enet0_gmii_tx_clk : IN STD_LOGIC;
		emio_enet0_gmii_txd : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		emio_enet0_gmii_tx_en : OUT STD_LOGIC;
		emio_enet0_gmii_tx_er : OUT STD_LOGIC;
		emio_enet0_mdio_mdc : OUT STD_LOGIC;
		emio_enet0_mdio_i : IN STD_LOGIC;
		emio_enet0_mdio_o : OUT STD_LOGIC;
		emio_enet0_mdio_t : OUT STD_LOGIC;
		emio_enet0_tsu_inc_ctrl : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		emio_enet0_tsu_timer_cmp_val : OUT STD_LOGIC;
		emio_enet0_enet_tsu_timer_cnt : OUT STD_LOGIC_VECTOR(93 DOWNTO 0);
		emio_enet0_ext_int_in : IN STD_LOGIC;
		emio_enet0_dma_bus_width : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		emio_gpio_i : IN STD_LOGIC_VECTOR(4 DOWNTO 0);
		emio_gpio_o : OUT STD_LOGIC_VECTOR(4 DOWNTO 0);
		emio_gpio_t : OUT STD_LOGIC_VECTOR(4 DOWNTO 0);
		emio_i2c0_scl_i : IN STD_LOGIC;
		emio_i2c0_scl_o : OUT STD_LOGIC;
		emio_i2c0_scl_t : OUT STD_LOGIC;
		emio_i2c0_sda_i : IN STD_LOGIC;
		emio_i2c0_sda_o : OUT STD_LOGIC;
		emio_i2c0_sda_t : OUT STD_LOGIC;
		emio_i2c1_scl_i : IN STD_LOGIC;
		emio_i2c1_scl_o : OUT STD_LOGIC;
		emio_i2c1_scl_t : OUT STD_LOGIC;
		emio_i2c1_sda_i : IN STD_LOGIC;
		emio_i2c1_sda_o : OUT STD_LOGIC;
		emio_i2c1_sda_t : OUT STD_LOGIC;
		pl_ps_irq0 : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
		pl_resetn0 : OUT STD_LOGIC;
		pl_clk0 : OUT STD_LOGIC
	  );
	end component design_1_zynq_ultra_ps_e_0_0;
	
	component design_1_axi_iic_0_0 is
	port (
	    s_axi_aclk : IN STD_LOGIC;
        s_axi_aresetn : IN STD_LOGIC;
        iic2intc_irpt : OUT STD_LOGIC;
        s_axi_awaddr : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
        s_axi_awvalid : IN STD_LOGIC;
        s_axi_awready : OUT STD_LOGIC;
        s_axi_wdata : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
        s_axi_wstrb : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
        s_axi_wvalid : IN STD_LOGIC;
        s_axi_wready : OUT STD_LOGIC;
        s_axi_bresp : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
        s_axi_bvalid : OUT STD_LOGIC;
        s_axi_bready : IN STD_LOGIC;
        s_axi_araddr : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
        s_axi_arvalid : IN STD_LOGIC;
        s_axi_arready : OUT STD_LOGIC;
        s_axi_rdata : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
        s_axi_rresp : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
        s_axi_rvalid : OUT STD_LOGIC;
        s_axi_rready : IN STD_LOGIC;
        sda_i : IN STD_LOGIC;
        sda_o : OUT STD_LOGIC;
        sda_t : OUT STD_LOGIC;
        scl_i : IN STD_LOGIC;
        scl_o : OUT STD_LOGIC;
        scl_t : OUT STD_LOGIC;
        gpo : OUT STD_LOGIC_VECTOR(0 DOWNTO 0)
	);
	end component design_1_axi_iic_0_0;
	
	component design_1_gig_ethernet_pcs_pma_0_0 is
	generic (
	  EXAMPLE_SIMULATION       : integer   := 0
	);
    port (
        gtrefclk_p               : in  std_logic;                         
        gtrefclk_n               : in  std_logic;                         
        gtrefclk_out             : out std_logic;                         
        txp                      : out std_logic;                    
        txn                      : out std_logic;                    
        rxp                      : in std_logic;                     
        rxn                      : in std_logic;                     
        userclk_out              : out std_logic;                    
        userclk2_out             : out std_logic;                    
        rxuserclk_out            : out std_logic;                    
        rxuserclk2_out           : out std_logic;                    
        pma_reset_out            : out std_logic;                    
        mmcm_locked_out          : out std_logic;                    
        resetdone                : out std_logic;
        independent_clock_bufg   : in std_logic;                
        gmii_txclk               : out std_logic;                   
        gmii_rxclk               : out std_logic;                   
        gmii_txd                 : in std_logic_vector(7 downto 0);  
        gmii_tx_en               : in std_logic;                     
        gmii_tx_er               : in std_logic;                     
        gmii_rxd                 : out std_logic_vector(7 downto 0); 
        gmii_rx_dv               : out std_logic;                    
        gmii_rx_er               : out std_logic;                    
        gmii_isolate             : out std_logic;                    
        mdc                      : in std_logic;                     
        mdio_i                   : in std_logic;                     
        mdio_o                   : out std_logic;                    
        mdio_t                   : out std_logic;                    
        phyaddr                  : in std_logic_vector(4 downto 0); 
        configuration_vector     : in std_logic_vector(4 downto 0);  
        configuration_valid      : in std_logic;                     
        an_interrupt             : out std_logic;                    
        an_adv_config_vector     : in std_logic_vector(15 downto 0); 
        an_adv_config_val        : in std_logic;                     
        an_restart_config        : in std_logic;                     
        status_vector            : out std_logic_vector(15 downto 0);
        reset                    : in std_logic;                     
        gtpowergood              : out std_logic;
        signal_detect            : in std_logic                      
    );
    end component design_1_gig_ethernet_pcs_pma_0_0;
    
    component design_1_rst_ps8_0_50M_0 is
    port (
        slowest_sync_clk : IN STD_LOGIC;
        ext_reset_in : IN STD_LOGIC;
        aux_reset_in : IN STD_LOGIC;
        mb_debug_sys_rst : IN STD_LOGIC;
        dcm_locked : IN STD_LOGIC;
        mb_reset : OUT STD_LOGIC;
        bus_struct_reset : OUT STD_LOGIC_VECTOR(0 DOWNTO 0);
        peripheral_reset : OUT STD_LOGIC_VECTOR(0 DOWNTO 0);
        interconnect_aresetn : OUT STD_LOGIC_VECTOR(0 DOWNTO 0);
        peripheral_aresetn : OUT STD_LOGIC_VECTOR(0 DOWNTO 0)
    );
    end component design_1_rst_ps8_0_50M_0;
    

-----------------------------------------------------------------------
-- Signals
-----------------------------------------------------------------------

    signal pl_clk_i             : std_logic;
    signal pl_resetn_i          : std_logic;
    signal aresetn              : std_logic_vector(0 downto 0);
    signal iic2intc_irpt        : std_logic_vector(0 downto 0);

	signal emio_i2c0_scl_i      : STD_LOGIC;
	signal emio_i2c0_scl_o      : STD_LOGIC;
	signal emio_i2c0_scl_t      : STD_LOGIC;
	signal emio_i2c0_sda_i      : STD_LOGIC;
	signal emio_i2c0_sda_o      : STD_LOGIC;
	signal emio_i2c0_sda_t      : STD_LOGIC;
	signal emio_i2c1_scl_i      : STD_LOGIC;
	signal emio_i2c1_scl_o      : STD_LOGIC;
	signal emio_i2c1_scl_t      : STD_LOGIC;
	signal emio_i2c1_sda_i      : STD_LOGIC;
	signal emio_i2c1_sda_o      : STD_LOGIC;
	signal emio_i2c1_sda_t      : STD_LOGIC;

    signal status_vector        : std_logic_vector(15 downto 0);
    signal mdc                  : std_logic;
    signal mdio_i               : std_logic;                     
    signal mdio_o               : std_logic;                    
    signal mdio_t               : std_logic;
    signal emio_enet0_mdio_t    : std_logic;
    signal rst 	                : std_logic;

    signal an_config            : std_logic;
    signal an_config_vector     : std_logic_vector(15 downto 0);
    signal emio_gpio_o          : std_logic_vector(4 downto 0);
    signal emio_gpio_i          : std_logic_vector(4 downto 0);
    signal ps_i2c1_sda_i        : std_logic;
    signal ps_i2c1_sda_t_i      : std_logic;
    signal ps_i2c1_sda_o        : std_logic;
    signal ps_i2c1_scl_i        : std_logic;
    signal ps_i2c1_scl_t_i      : std_logic;
    signal ps_i2c1_scl_o        : std_logic;
    signal board_sda_i          : std_logic;
    signal board_sda_o          : std_logic;
    signal board_sda_t_o        : std_logic;
    signal board_scl_i          : std_logic;
    signal board_scl_o          : std_logic;
    signal board_scl_t_o        : std_logic;

	signal phyaddr              : std_logic_vector(4 downto 0);
    signal configuration_vector : std_logic_vector(4 downto 0);
    signal configuration_valid  : std_logic_vector(0 downto 0);
    signal signal_detect        : std_logic_vector(0 downto 0);  

    signal gmii_txclk           : std_logic;                    
    signal gmii_rxclk           : std_logic;                    
    signal gmii_txd             : std_logic_vector(7 downto 0);  
    signal gmii_tx_en           : std_logic;                     
    signal gmii_tx_er           : std_logic;                     
    signal gmii_rxd             : std_logic_vector(7 downto 0); 
    signal gmii_rx_dv           : std_logic;                    
    signal gmii_rx_er           : std_logic;                    
    signal gmii_isolate         : std_logic;      

    signal sda_i                : std_logic;
    signal sda_o                : std_logic;
    signal sda_t                : std_logic;
    signal scl_i                : std_logic;
    signal scl_o                : std_logic;
    signal scl_t                : std_logic;  
    
    signal s00_axi_awid         : STD_LOGIC_VECTOR ( 15 downto 0 );
    signal s00_axi_awaddr       : STD_LOGIC_VECTOR ( 39 downto 0 );
    signal s00_axi_awlen        : STD_LOGIC_VECTOR ( 7 downto 0 );
    signal s00_axi_awsize       : STD_LOGIC_VECTOR ( 2 downto 0 );
    signal s00_axi_awburst      : STD_LOGIC_VECTOR ( 1 downto 0 );
    signal s00_axi_awlock       : STD_LOGIC_VECTOR ( 0 to 0 );
    signal s00_axi_awcache      : STD_LOGIC_VECTOR ( 3 downto 0 );
    signal s00_axi_awprot       : STD_LOGIC_VECTOR ( 2 downto 0 );
    signal s00_axi_awvalid      : STD_LOGIC;
    signal s00_axi_awuser       : STD_LOGIC_VECTOR(15 DOWNTO 0);
    signal s00_axi_awready      : STD_LOGIC;
    signal s00_axi_wdata        : STD_LOGIC_VECTOR ( 31 downto 0 );
    signal s00_axi_wstrb        : STD_LOGIC_VECTOR ( 3 downto 0 );
    signal s00_axi_wlast        : STD_LOGIC;
    signal s00_axi_wvalid       : STD_LOGIC;
    signal s00_axi_wready       : STD_LOGIC;
    signal s00_axi_bid          : STD_LOGIC_VECTOR ( 15 downto 0 );
    signal s00_axi_bresp        : STD_LOGIC_VECTOR ( 1 downto 0 );
    signal s00_axi_bvalid       : STD_LOGIC;
    signal s00_axi_bready       : STD_LOGIC;
    signal s00_axi_arid         : STD_LOGIC_VECTOR ( 15 downto 0 );
    signal s00_axi_araddr       : STD_LOGIC_VECTOR ( 39 downto 0 );
    signal s00_axi_arlen        : STD_LOGIC_VECTOR ( 7 downto 0 );
    signal s00_axi_arsize       : STD_LOGIC_VECTOR ( 2 downto 0 );
    signal s00_axi_arburst      : STD_LOGIC_VECTOR ( 1 downto 0 );
    signal s00_axi_arlock       : STD_LOGIC_VECTOR ( 0 to 0 );
    signal s00_axi_arcache      : STD_LOGIC_VECTOR ( 3 downto 0 );
    signal s00_axi_arprot       : STD_LOGIC_VECTOR ( 2 downto 0 );
    signal s00_axi_arvalid      : STD_LOGIC;
    signal s00_axi_aruser       : STD_LOGIC_VECTOR(15 DOWNTO 0);
    signal s00_axi_arready      : STD_LOGIC;
    signal s00_axi_rid          : STD_LOGIC_VECTOR ( 15 downto 0 );
    signal s00_axi_rdata        : STD_LOGIC_VECTOR ( 31 downto 0 );
    signal s00_axi_rresp        : STD_LOGIC_VECTOR ( 1 downto 0 );
    signal s00_axi_rlast        : STD_LOGIC;
    signal s00_axi_rvalid       : STD_LOGIC;
    signal s00_axi_rready       : STD_LOGIC;
    signal s00_axi_awqos        : STD_LOGIC_VECTOR ( 3 downto 0 );
    signal s00_axi_arqos        : STD_LOGIC_VECTOR ( 3 downto 0 );
    
    signal M00_AXI_araddr       : STD_LOGIC_VECTOR ( 8 downto 0 );
    signal M00_AXI_arready      : STD_LOGIC;
    signal M00_AXI_arvalid      : STD_LOGIC;
    signal M00_AXI_awaddr       : STD_LOGIC_VECTOR ( 8 downto 0 );
    signal M00_AXI_awready      : STD_LOGIC;
    signal M00_AXI_awvalid      : STD_LOGIC;
    signal M00_AXI_bready       : STD_LOGIC;
    signal M00_AXI_bresp        : STD_LOGIC_VECTOR ( 1 downto 0 );
    signal M00_AXI_bvalid       : STD_LOGIC;
    signal M00_AXI_rdata        : STD_LOGIC_VECTOR ( 31 downto 0 );
    signal M00_AXI_rready       : STD_LOGIC;
    signal M00_AXI_rresp        : STD_LOGIC_VECTOR ( 1 downto 0 );
    signal M00_AXI_rvalid       : STD_LOGIC;
    signal M00_AXI_wdata        : STD_LOGIC_VECTOR ( 31 downto 0 );
    signal M00_AXI_wready       : STD_LOGIC;
    signal M00_AXI_wstrb        : STD_LOGIC_VECTOR ( 3 downto 0 );
    signal M00_AXI_wvalid       : STD_LOGIC;
    
    --AXI i2c xbar signals for Xilinx IP core AXI IIC
    signal  i2c_0_awvalid       : std_logic;
    signal  i2c_0_awready       : std_logic;
    signal  i2c_0_awaddr        : std_logic_vector(10 downto 2);
    signal  i2c_0_awprot        : std_logic_vector(2 downto 0);
    signal  i2c_0_wvalid        : std_logic;
    signal  i2c_0_wready        : std_logic;
    signal  i2c_0_wdata         : std_logic_vector(31 downto 0);
    signal  i2c_0_wstrb         : std_logic_vector(3 downto 0);
    signal  i2c_0_bvalid        : std_logic;
    signal  i2c_0_bready        : std_logic;
    signal  i2c_0_bresp         : std_logic_vector(1 downto 0);
    signal  i2c_0_arvalid       : std_logic;
    signal  i2c_0_arready       : std_logic;
    signal  i2c_0_araddr        : std_logic_vector(10 downto 2);
    signal  i2c_0_arprot        : std_logic_vector(2 downto 0);
    signal  i2c_0_rvalid        : std_logic;
    signal  i2c_0_rready        : std_logic;
    signal  i2c_0_rdata         : std_logic_vector(31 downto 0);
    signal  i2c_0_rresp         : std_logic_vector(1 downto 0);
    
    --AXI I2C Xilinx IP core signals
    signal i2c_axi_0_awready    : std_logic;
    signal i2c_axi_0_wready     : std_logic;
    signal i2c_axi_0_bresp      : std_logic_vector(1 downto 0);
    signal i2c_axi_0_bvalid     : std_logic;
    signal i2c_axi_0_arready    : std_logic;
    signal i2c_axi_0_rdata      : std_logic_vector(31 downto 0);
    signal i2c_axi_0_rresp      : std_logic_vector(1 downto 0);
    signal i2c_axi_0_rvalid     : std_logic;   

    -- Instead of assignin values directly to port map
    -- otherwise implementation is failing
    signal  emio_enet0_tsu_inc_ctrl : std_logic_vector(1 downto 0);
    signal  emio_enet0_gmii_crs : std_logic;
    signal  emio_enet0_gmii_col : std_logic;
    signal  emio_enet0_ext_int_in : std_logic;
    signal  aux_reset_in        : std_logic;   
    signal  mb_debug_sys_rst    : std_logic;
    signal  gpio_i              : std_logic_vector(4 downto 0);
    signal  gpio_o              : std_logic_vector(4 downto 0);
    
begin
	iobuffers : block
	-- signals declaration if needed
	begin
		inst_bufds_0: component IOBUF
		port map (
		I  => board_scl_o,
		IO => emio_i2c_scl(0),
		O  => board_scl_i,
		T  => board_scl_t_o
		);
		
		inst_bufds_1: component IOBUF
		port map (
		I  => board_sda_o,
		IO => emio_i2c_sda(0),
		O  => board_sda_i,
		T  => board_sda_t_o
		);
		
		inst_bufds_2: component IOBUF
		port map (
		I  => emio_i2c0_sda_o,
		IO => M_SDA_0(0),
		O  => emio_i2c0_sda_i,
		T  => emio_i2c0_sda_t
		);

		inst_bufds_3: component IOBUF
		port map (
		I  => emio_i2c0_scl_o,
		IO => M_SCL_0(0),
		O  => emio_i2c0_scl_i,
		T  => emio_i2c0_scl_t
		);

		inst_bufds_4: component IOBUF
		port map (
		I  => sda_o, 
		IO => wrflash_sda(0),
		O  => sda_i,
		T  => sda_t
		);

		inst_bufds_5: component IOBUF
		port map (
		I  => scl_o,
		IO => wrflash_scl(0),
		O  => scl_i,
		T  => scl_t
		);
	end block; --iobuffers

	-- Zynq MPSoC
	inst_zynq_us: design_1_zynq_ultra_ps_e_0_0
	port map ( 
	    maxihpm0_fpd_aclk             => pl_clk_i,
	    maxigp0_awid                  => s00_axi_awid,
	    maxigp0_awaddr                => s00_axi_awaddr,
	    maxigp0_awlen                 => s00_axi_awlen,
	    maxigp0_awsize                => s00_axi_awsize,     
	    maxigp0_awburst               => s00_axi_awburst,    
	    maxigp0_awlock                => s00_axi_awlock(0),     
	    maxigp0_awcache               => s00_axi_awcache,    
	    maxigp0_awprot                => s00_axi_awprot,     
	    maxigp0_awvalid               => s00_axi_awvalid,    
	    maxigp0_awuser                => s00_axi_awuser,     
	    maxigp0_awready               => s00_axi_awready,    
	    maxigp0_wdata                 => s00_axi_wdata,      
	    maxigp0_wstrb                 => s00_axi_wstrb,      
	    maxigp0_wlast                 => s00_axi_wlast,      
	    maxigp0_wvalid                => s00_axi_wvalid,     
	    maxigp0_wready                => s00_axi_wready,     
	    maxigp0_bid                   => s00_axi_bid,        
	    maxigp0_bresp                 => s00_axi_bresp,      
	    maxigp0_bvalid                => s00_axi_bvalid,     
	    maxigp0_bready                => s00_axi_bready,     
	    maxigp0_arid                  => s00_axi_arid,       
	    maxigp0_araddr                => s00_axi_araddr,     
	    maxigp0_arlen                 => s00_axi_arlen,      
	    maxigp0_arsize                => s00_axi_arsize,     
	    maxigp0_arburst               => s00_axi_arburst,    
	    maxigp0_arlock                => s00_axi_arlock(0),     
		maxigp0_arcache               => s00_axi_arcache,       
		maxigp0_arprot                => s00_axi_arprot,        
		maxigp0_arvalid               => s00_axi_arvalid,       
		maxigp0_aruser                => s00_axi_aruser,        
		maxigp0_arready               => s00_axi_arready,       
		maxigp0_rid                   => s00_axi_rid,           
		maxigp0_rdata                 => s00_axi_rdata,         
		maxigp0_rresp                 => s00_axi_rresp,         
		maxigp0_rlast                 => s00_axi_rlast,         
		maxigp0_rvalid                => s00_axi_rvalid,        
		maxigp0_rready                => s00_axi_rready,        
		maxigp0_awqos                 => s00_axi_awqos,         
		maxigp0_arqos                 => s00_axi_arqos, 
		emio_enet0_gmii_rx_clk        => gmii_rxclk,
		emio_enet0_speed_mode         => open, 
		emio_enet0_gmii_crs           => emio_enet0_gmii_crs,--'0', --carrier sense
		emio_enet0_gmii_col           => emio_enet0_gmii_col,--'0', --collision detect
		emio_enet0_gmii_rxd           => gmii_rxd, 
		emio_enet0_gmii_rx_er         => gmii_rx_er,
		emio_enet0_gmii_rx_dv         => gmii_rx_dv, 
		emio_enet0_gmii_tx_clk        => gmii_txclk,
		emio_enet0_gmii_txd           => gmii_txd,
		emio_enet0_gmii_tx_en         => gmii_tx_en,
		emio_enet0_gmii_tx_er         => gmii_tx_er,
		emio_enet0_mdio_mdc           => mdc,
		emio_enet0_mdio_i             => mdio_o,
		emio_enet0_mdio_o             => mdio_i,
		emio_enet0_mdio_t             => emio_enet0_mdio_t,
		emio_enet0_tsu_inc_ctrl       => emio_enet0_tsu_inc_ctrl,--"00",
		emio_enet0_tsu_timer_cmp_val  => open,
		emio_enet0_enet_tsu_timer_cnt => open,
		emio_enet0_ext_int_in         => emio_enet0_ext_int_in, --'0',
		emio_enet0_dma_bus_width      => open,
		emio_gpio_i                   => gpio_o,
		emio_gpio_o                   => gpio_i,
		emio_gpio_t                   => open,
		emio_i2c0_scl_i               => emio_i2c0_scl_i,
		emio_i2c0_scl_o               => emio_i2c0_scl_o,
		emio_i2c0_scl_t               => emio_i2c0_scl_t,
		emio_i2c0_sda_i               => emio_i2c0_sda_i,
		emio_i2c0_sda_o               => emio_i2c0_sda_o,
		emio_i2c0_sda_t               => emio_i2c0_sda_t,
		emio_i2c1_scl_i               => ps_i2c1_scl_o,
		emio_i2c1_scl_o               => ps_i2c1_scl_i,
		emio_i2c1_scl_t               => ps_i2c1_scl_t_i,
		emio_i2c1_sda_i               => ps_i2c1_sda_o, 
		emio_i2c1_sda_o               => ps_i2c1_sda_i,
		emio_i2c1_sda_t               => ps_i2c1_sda_t_i,
		pl_ps_irq0                    => iic2intc_irpt,
		pl_resetn0                    => pl_resetn_i, 
		pl_clk0                       => pl_clk_i
	);

  -- Constants.vhd
  inst_constants: entity work.constants
  port map (
        pl_clk_i             => pl_clk_i,
        status_vector        => status_vector, 
        mdc_i                => mdc, 
  		mdc_clk_led 	     => mdc_clk_led(0),
        pl_resetn_i          => pl_resetn_i,
        reset_o 	         => rst, 
        an_config_o          => an_config, 
        an_config_vec_o      => an_config_vector,
        p_pres_i             => p_pres_i_0, 
        pwr_cycle_req_o      => pwr_cycle_req_o_0, 
        ps_emio_o            => gpio_o, 
        ps_emio_i            => gpio_i,  
        ps_i2c1_sda_i        => ps_i2c1_sda_i, 
        ps_i2c1_sda_t_i      => ps_i2c1_sda_t_i,
        ps_i2c1_sda_o        => ps_i2c1_sda_o,
        ps_i2c1_scl_i        => ps_i2c1_scl_i,
        ps_i2c1_scl_t_i      => ps_i2c1_scl_t_i,
        ps_i2c1_scl_o        => ps_i2c1_scl_o, 
        board_sda_i          => board_sda_i, 
        board_sda_o          => board_sda_o, 
        board_sda_t_o        => board_sda_t_o,
        board_scl_i          => board_scl_i,
        board_scl_o          => board_scl_o, 
        board_scl_t_o        => board_scl_t_o,
	    phyaddr              => phyaddr, 
        configuration_vector => configuration_vector,
        configuration_valid  => configuration_valid,
        signal_detect        => signal_detect,
        link_status_led      => link_status_led(0), 
        link_sync_led        => link_sync_led(0)
  );

  -- 1G/2.5G Ethernet PCS/PMA
  inst_gig_ethernet_pcs_pma: design_1_gig_ethernet_pcs_pma_0_0
  generic map (
       EXAMPLE_SIMULATION    => 0
  )
  port map (
      gtrefclk_p             => gtrefclk_in_clk_p,
      gtrefclk_n             => gtrefclk_in_clk_n,
      gtrefclk_out           => open,
      txp                    => sfp_txp,
      txn                    => sfp_txn, 
      rxp                    => sfp_rxp,
      rxn                    => sfp_rxn, 
      resetdone              => open,
      userclk_out            => open,
      userclk2_out           => open,
      rxuserclk_out          => open,
      rxuserclk2_out         => open,
      pma_reset_out          => open,
      mmcm_locked_out        => open,
      independent_clock_bufg => pl_clk_i, 
      gmii_txclk             => gmii_txclk, 
      gmii_rxclk             => gmii_rxclk,
      gmii_txd               => gmii_txd,
      gmii_tx_en             => gmii_tx_en,
      gmii_tx_er             => gmii_tx_er,
      gmii_rxd               => gmii_rxd,
      gmii_rx_dv             => gmii_rx_dv,
      gmii_rx_er             => gmii_rx_er,
      gmii_isolate           => open,
      mdc                    => mdc, 
      mdio_i                 => mdio_i,
      mdio_o                 => mdio_o,
      mdio_t                 => mdio_t,
      phyaddr                => phyaddr,
      configuration_vector   => configuration_vector,
      configuration_valid    => configuration_valid(0),
      an_interrupt           => open,
      an_adv_config_vector   => an_config_vector,
      an_adv_config_val      => an_config,
      an_restart_config      => an_config,
      status_vector          => status_vector,
      reset                  => rst,
      gtpowergood            => open,
      signal_detect          => signal_detect(0)
  );

  -- Processor System Reset IP core
  inst_proc_system_reset: design_1_rst_ps8_0_50M_0
  port map (
    slowest_sync_clk     => pl_clk_i,
    ext_reset_in         => pl_resetn_i,
    aux_reset_in         => '0', --set the logic level for which aux_reset_in causes a reset
    mb_debug_sys_rst     => '0',
    dcm_locked           => '1', --if the system does not use any DCMs this input should be tied High.
    mb_reset             => open,
    bus_struct_reset     => open,
    peripheral_reset     => open,
    interconnect_aresetn => open,
    peripheral_aresetn   => aresetn 
  );

  -- This component taken from General Cores
  -- in order to replace the AXI Interconnect IP core from Xilinx
  inst_axi4lite_to_axi4full: entity work.axi4lite32_axi4full64_bridge
  port map (
    clk_i     => pl_clk_i, 
    rst_n_i   => aresetn(0),
    --  AXI4-Full slave
    s_awaddr  => s00_axi_awaddr,
    s_awlen   => s00_axi_awlen,
    s_awsize  => s00_axi_awsize,
    s_awburst => s00_axi_awburst,
    s_awvalid => s00_axi_awvalid,
    s_awready => s00_axi_awready,    
    s_wdata   => s00_axi_wdata,
    s_wstrb   => s00_axi_wstrb,
    s_wlast   => s00_axi_wlast,
    s_wvalid  => s00_axi_wvalid,
    s_wready  => s00_axi_wready,
    s_bresp   => s00_axi_bresp,
    s_bvalid  => s00_axi_bvalid,
    s_bready  => s00_axi_bready,
    s_araddr  => s00_axi_araddr,
    s_arlen   => s00_axi_arlen,
    s_arsize  => s00_axi_arsize,
    s_arburst => s00_axi_arburst,
    s_arvalid => s00_axi_arvalid,
    s_arready => s00_axi_arready,
    s_rdata   => s00_axi_rdata,
    s_rresp   => s00_axi_rresp,
    s_rlast   => s00_axi_rlast,
    s_rvalid  => s00_axi_rvalid,
    s_rready  => s00_axi_rready,
    s_awid    => open, --s00_axi_awid,
    s_awlock  => open, --s00_axi_awlock,
    s_awcache => open, --s00_axi_awcache,
    s_awprot  => open, --s00_axi_awprot,
    s_awuser  => open, --s00_axi_awuser,
    s_awqos   => open, --s00_axi_awqos,
    s_bid     => s00_axi_bid,
    s_arid    => open, --s00_axi_arid,
    s_arlock  => open, --s00_axi_arlock,
    s_arcache => open, --s00_axi_arcache,
    s_arprot  => open, --s00_axi_arprot,
    s_aruser  => open, --s00_axi_aruser,
    s_arqos   => open, --s00_axi_arqos,
    s_rid     => s00_axi_rid,
    --  AXI4-Lite master
    m_awaddr  => m00_axi_awaddr,
    m_awvalid => m00_axi_awvalid,
    m_awready => m00_axi_awready,
    m_wdata   => m00_axi_wdata,
    m_wstrb   => m00_axi_wstrb,
    m_wvalid  => m00_axi_wvalid,
    m_wready  => m00_axi_wready,
    m_bresp   => m00_axi_bresp,
    m_bvalid  => m00_axi_bvalid,
    m_bready  => m00_axi_bready,
    m_araddr  => m00_axi_araddr,
    m_arvalid => m00_axi_arvalid,
    m_arready => m00_axi_arready,
    m_rdata   => m00_axi_rdata,
    m_rresp   => m00_axi_rresp,
    m_rvalid  => m00_axi_rvalid,
    m_rready  => m00_axi_rready
  );
  
  --AXI I2C block(s)
  axi_i2c_blocks : block
  -- signals declaration if needed
  begin
      inst_axi_iic_0: component design_1_axi_iic_0_0
      port map (
        s_axi_aclk    => pl_clk_i, 
        s_axi_aresetn => aresetn(0),
        iic2intc_irpt => iic2intc_irpt(0), 
        s_axi_awaddr  => i2c_0_awaddr,
        s_axi_awvalid => i2c_0_awvalid,
        s_axi_awready => i2c_axi_0_awready,
        s_axi_wdata   => i2c_0_wdata,
        s_axi_wstrb   => i2c_0_wstrb,
        s_axi_wvalid  => i2c_0_wvalid,
        s_axi_wready  => i2c_axi_0_wready,
        s_axi_bresp   => i2c_axi_0_bresp,
        s_axi_bvalid  => i2c_axi_0_bvalid,
        s_axi_bready  => i2c_0_bready,
        s_axi_araddr  => i2c_0_araddr,
        s_axi_arvalid => i2c_0_arvalid,
        s_axi_arready => i2c_axi_0_arready,
        s_axi_rdata   => i2c_axi_0_rdata,
        s_axi_rresp   => i2c_axi_0_rresp,
        s_axi_rvalid  => i2c_axi_0_rvalid,
        s_axi_rready  => i2c_0_rready,
        sda_i         => sda_i,
        sda_o         => sda_o,
        sda_t         => sda_t,
        scl_i         => scl_i,
        scl_o         => scl_o,
        scl_t         => scl_t,
        gpo           => open
      );
  end block; --axi i2c blocks
	
  inst_diot_sb_xbar: entity work.diot_sb_xbar
  port map (
      aclk            => pl_clk_i,   
      areset_n        => aresetn(0),    
      awvalid         => m00_axi_awvalid, 
      awready         => m00_axi_awready,    
      awaddr          => m00_axi_awaddr,    
      awprot          => "000",     --this signal doesn't exist in general core's IP core    
      wvalid          => m00_axi_wvalid,    
      wready          => m00_axi_wready,    
      wdata           => m00_axi_wdata,    
      wstrb           => m00_axi_wstrb,    
      bvalid          => m00_axi_bvalid,    
      bready          => m00_axi_bready,    
      bresp           => m00_axi_bresp, 
      arvalid         => m00_axi_arvalid,
      arready         => m00_axi_arready,
      araddr          => m00_axi_araddr,
      arprot          => "000",     --this signal doesn't exist in general core's IP core
      rvalid          => m00_axi_rvalid,
      rready          => m00_axi_rready,
      rdata           => m00_axi_rdata,
      rresp           => m00_axi_rresp,

      -- AXI-4 lite bus i2c_0
      i2c_0_awvalid_o => i2c_0_awvalid,
      i2c_0_awready_i => i2c_axi_0_awready,
      i2c_0_awaddr_o  => i2c_0_awaddr,    
      i2c_0_awprot_o  => i2c_0_awprot,
      i2c_0_wvalid_o  => i2c_0_wvalid,
      i2c_0_wready_i  => i2c_axi_0_wready,
      i2c_0_wdata_o   => i2c_0_wdata,    
      i2c_0_wstrb_o   => i2c_0_wstrb,    
      i2c_0_bvalid_i  => i2c_axi_0_bvalid,
      i2c_0_bready_o  => i2c_0_bready,    
      i2c_0_bresp_i   => i2c_axi_0_bresp,
      i2c_0_arvalid_o => i2c_0_arvalid,    
      i2c_0_arready_i => i2c_axi_0_arready,
      i2c_0_araddr_o  => i2c_0_araddr,    
      i2c_0_arprot_o  => i2c_0_arprot,
      i2c_0_rvalid_i  => i2c_axi_0_rvalid,
      i2c_0_rready_o  => i2c_0_rready,   
      i2c_0_rdata_i   => i2c_axi_0_rdata,
      i2c_0_rresp_i   => i2c_axi_0_rresp 
    );
                                     

    pl_reset_led <= pl_resetn_i;

end arch;