/*
 * diot_util - diagnostic tool for DIOT system board
 * 
 * Author: Adam Wujek for CERN
 * Copyright CERN 2021
 * 
 */


#include <stdio.h>
#include <string.h>
#include <gpiod.h>
#include <unistd.h>
#include <errno.h>

#include <readline/readline.h>
#include <readline/history.h>

const char *chipname = "gpiochip0";
const char *chiplabel = "zynqmp_gpio";
struct gpiod_chip *chip;

#define	CONSUMER	"diot_util"
#define READLINE_PROMPT "diot_util> "

typedef enum {
    dir_in,
    dir_out
} dir_t;

struct gpio_entry {
    char *gpio_name;
    struct gpiod_line *gpio_line;
    dir_t dir;
    int def_out;
};

struct command_entry {
    char *cmd_name;
    int (*cmd_func)(char *params);
    char *cmd_help_string;
    char *cmd_params;
};

enum {
    USER_LED2,
    SYSEN_N,
    FMC_PRSNT,
    PCB_VER0,
    PCB_VER1,
    PCB_VER2,
    PCB_VER3,
    PWR_Cycle_REQ,
    PWR_Cycle_STR,
    P_PRES0,
    P_PRES1,
    SERVMOD_N1,
    SERVMOD_N2,
    SERVMOD_N3,
    SERVMOD_N4,
    SERVMOD_N5,
    SERVMOD_N6,
    SERVMOD_N7,
    SERVMOD_N8,
    RST_N,
    PSU_ALERT,
    MAX_GPIO,
};

#define DECLARE_GPIO_IN(pin) [pin] = { .gpio_name = #pin, .dir = dir_in }
#define DECLARE_GPIO_OUT(pin, default_state) [pin] = { .gpio_name = #pin, .dir = dir_out, .def_out = default_state }

struct gpio_entry gpio_entries[MAX_GPIO] = {
    DECLARE_GPIO_OUT(USER_LED2, 1),
    DECLARE_GPIO_IN(SYSEN_N),
    DECLARE_GPIO_IN(FMC_PRSNT),
    DECLARE_GPIO_IN(PCB_VER0),
    DECLARE_GPIO_IN(PCB_VER1),
    DECLARE_GPIO_IN(PCB_VER2),
    DECLARE_GPIO_IN(PCB_VER3),
    DECLARE_GPIO_OUT(PWR_Cycle_REQ, 1),
    DECLARE_GPIO_OUT(PWR_Cycle_STR, 1),
    DECLARE_GPIO_IN(P_PRES0),
    DECLARE_GPIO_IN(P_PRES1),
    DECLARE_GPIO_IN(SERVMOD_N1),
    DECLARE_GPIO_IN(SERVMOD_N2),
    DECLARE_GPIO_IN(SERVMOD_N3),
    DECLARE_GPIO_IN(SERVMOD_N4),
    DECLARE_GPIO_IN(SERVMOD_N5),
    DECLARE_GPIO_IN(SERVMOD_N6),
    DECLARE_GPIO_IN(SERVMOD_N7),
    DECLARE_GPIO_IN(SERVMOD_N8),
    DECLARE_GPIO_OUT(RST_N, 1),
    DECLARE_GPIO_IN(PSU_ALERT),
};

/* readline related */
char **command_completion(const char *, int, int);
char *command_name_generator(const char *, int);

/* commands */
int cmd_print_status(char *params);
int cmd_exit(char *params);
int cmd_help(char *params);
int cmd_led2(char *params);
int cmd_peripheral_rst(char *params);
int cmd_power_cycle(char *params);

int close_gpios(void);


struct command_entry commands_list[] = {
    { .cmd_name = "status",            .cmd_func = cmd_print_status,   .cmd_help_string = "Print status\n",             .cmd_params = NULL},
    { .cmd_name = "exit",              .cmd_func = cmd_exit,           .cmd_help_string = "quit program\n",             .cmd_params = NULL},
    { .cmd_name = "quit",              .cmd_func = cmd_exit,           .cmd_help_string = "quit program\n",             .cmd_params = NULL},
    { .cmd_name = "help",              .cmd_func = cmd_help,           .cmd_help_string = "Print help\n",               .cmd_params = NULL},
    { .cmd_name = "led2",              .cmd_func = cmd_led2,           .cmd_help_string = "Enable/disable LED2\n",      .cmd_params = "on|off"},
    { .cmd_name = "peripheral_reset",  .cmd_func = cmd_peripheral_rst, .cmd_help_string = "Reset peripheral boards\n",  .cmd_params = NULL},
    { .cmd_name = "power_cycle_crate", .cmd_func = cmd_power_cycle,    .cmd_help_string = "Power cycle DI/OT create\n", .cmd_params = NULL},
    { .cmd_name = NULL }
};

int cmd_help(char *params)
{
    struct command_entry *entry = commands_list;
    char buf[50];

    printf("Avaiable commands:\n");
    while (entry->cmd_name) {
	sprintf(buf, "%s %s", entry->cmd_name,
		entry->cmd_params ? entry->cmd_params : "");
	printf("%-15s - %s", buf, entry->cmd_help_string);
	entry++;
    }

    return 0;
}

int cmd_peripheral_rst(char *params)
{
    int ret;

    printf("Reseting peripheral boards...\n");
    ret = gpiod_line_set_value(gpio_entries[RST_N].gpio_line, 0);
    if (ret < 0)
	return ret;
    usleep(1000);
    ret = gpiod_line_set_value(gpio_entries[RST_N].gpio_line, 1);
    if (ret < 0)
	return ret;

    return 0;
}

int cmd_power_cycle(char *params)
{
    int ret;

    printf("Power cycle...\n");
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_REQ].gpio_line, 1);
    if (ret < 0)
	return ret;
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_STR].gpio_line, 0);
    if (ret < 0)
	return ret;
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_STR].gpio_line, 1);
    if (ret < 0)
	return ret;
    usleep(1000);
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_STR].gpio_line, 0);
    if (ret < 0)
	return ret;
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_REQ].gpio_line, 0);
    if (ret < 0)
	return ret;
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_STR].gpio_line, 1);
    if (ret < 0)
	return ret;
    usleep(1000);
    ret = gpiod_line_set_value(gpio_entries[PWR_Cycle_STR].gpio_line, 0);
    if (ret < 0)
	return ret;

    return 0;

}

int cmd_exit(char *params)
{
    printf("Exiting...\n");

    close_gpios();
    gpiod_chip_close(chip);
    exit(0);
}

int cmd_led2(char *params)
{
    int val = 0, ret;

    if (!strncmp(params, "on", 2) || !strncmp(params, "1", 1)) {
	printf("LED2 on\n");
	val = 1;
    } else if (!strncmp(params, "off", 3) || !strncmp(params, "0", 1)) {
	printf("LED2 off\n");
	val = 0;
    } else {
	printf("Wrong parameter\n");
	errno = EINVAL;
	return -1;
    }

    ret = gpiod_line_set_value(gpio_entries[USER_LED2].gpio_line, val);

    return 0;

}

int open_gpios(void)
{
    int i;
    struct gpiod_line *tmp_line;
    int ret;

    for (i = 0; i < MAX_GPIO; i++) {
	if (!gpio_entries[i].gpio_name)
	    continue;

	tmp_line = gpiod_chip_find_line(chip, gpio_entries[i].gpio_name);
	if (!tmp_line) {
	    printf("Error for line %s (%d)\n", gpio_entries[i].gpio_name, i);
	    return -1;
	}
	gpio_entries[i].gpio_line = tmp_line;
	if (gpio_entries[i].dir == dir_in) {
	    ret = gpiod_line_request_input(tmp_line, CONSUMER);
	    if (ret < 0) {
		    printf("Request line as %s (%d) input failed\n",
			gpio_entries[i].gpio_name, i);
		    return ret;
	    }
	} else if (gpio_entries[i].dir == dir_out) {
		ret = gpiod_line_request_output(tmp_line, CONSUMER, gpio_entries[i].def_out);
	    if (ret < 0) {
		    printf("Request line as %s (%d) input failed\n",
			gpio_entries[i].gpio_name, i);
		    return ret;
	    }
	}
    }

    return 0;
}

int close_gpios(void)
{
    int i;

    for (i = 0; i < MAX_GPIO; i++) {
	if (!gpio_entries[i].gpio_name)
	    continue;

	gpiod_line_release(gpio_entries[i].gpio_line);
	gpio_entries[i].gpio_line = NULL;
    }

    return 0;
}

int read_pcb_ver()
{
    int pcb_version = 0;

    pcb_version = gpiod_line_get_value(gpio_entries[PCB_VER3].gpio_line);
    pcb_version <<= 1;
    pcb_version = gpiod_line_get_value(gpio_entries[PCB_VER2].gpio_line);
    pcb_version <<= 1;
    pcb_version = gpiod_line_get_value(gpio_entries[PCB_VER1].gpio_line);
    pcb_version <<= 1;
    pcb_version = gpiod_line_get_value(gpio_entries[PCB_VER0].gpio_line);

    return pcb_version;
}

int read_mezzanine_presence()
{
    /* pin is low when FMC card is present */
    return !gpiod_line_get_value(gpio_entries[FMC_PRSNT].gpio_line);
}

int read_psu_presence(int psu)
{
    int psi_pin;

    if (psu == 0)
	psi_pin = P_PRES0;
    else if (psu == 1)
	psi_pin = P_PRES1;
    else
	return -1;

    return !gpiod_line_get_value(gpio_entries[psi_pin].gpio_line);
}

int read_slot_use(int slot)
{
    int slot_pin;

    switch (slot) {
	case 1:
	    slot_pin = SERVMOD_N1;
	    break;
	case 2:
	    slot_pin = SERVMOD_N2;
	    break;
	case 3:
	    slot_pin = SERVMOD_N3;
	    break;
	case 4:
	    slot_pin = SERVMOD_N4;
	    break;
	case 5:
	    slot_pin = SERVMOD_N5;
	    break;
	case 6:
	    slot_pin = SERVMOD_N6;
	    break;
	case 7:
	    slot_pin = SERVMOD_N7;
	    break;
	case 8:
	    slot_pin = SERVMOD_N8;
	    break;
	default:
	    return -1;
    }

    return gpiod_line_get_value(gpio_entries[slot_pin].gpio_line);
}


int read_system_slot_presence()
{
    return !gpiod_line_get_value(gpio_entries[SYSEN_N].gpio_line);
}

int cmd_print_status(char *params)
{
    int i;

    printf("PCB version: %d\n", read_pcb_ver());
    printf("FMC mezzanine: %s\n", read_mezzanine_presence() ? "present" : "not present");
    printf("PSU 0: %15s, PSU 1: %15s\n", read_psu_presence(0) ? "present" : "not present",
					 read_psu_presence(1) ? "present" : "not present");
    printf("Card in system slot: %s\n", read_system_slot_presence() ? "true" : "false");
    printf("slot: 1 2 3 4 5 6 7 8\n");
    printf("used:");
    for (i = 1; i <= 8; i++) {
	printf(" %s", read_slot_use(i)? "Y": "N");
    }
    printf("\n");
    return 1;
}

int main(int argc, char **argv)
{
    int val;
    struct command_entry *entry = commands_list;

    int i, ret;
    char *line_read = NULL;
    int cmd_len;
    char *params;
    
    printf("DIOT diagnostic tool\n");
    chip = gpiod_chip_open_by_label(chiplabel);

    if (!chip) {
	    perror("Open chip failed\n");
	    exit (1);
    }

    ret = open_gpios();
    if (ret < 0) {
	gpiod_chip_close(chip);
    }

    rl_attempted_completion_function = command_completion;
    rl_completer_word_break_characters = " ";

    while (1) {
	/* If the buffer has already been allocated, return the memory
	to the free pool. */
	if (line_read) {
	    free (line_read);
	    line_read = (char *)NULL;
	}

	/* Get a line from the user. */
	line_read = readline (READLINE_PROMPT);

	/* If the line has any text in it, save it on the history. */
	if (line_read && *line_read)
	    add_history (line_read);
	else
	    continue;
	
	params = strchr(line_read, ' ');
	if (params) {
	    cmd_len = params - line_read;
	    /* skip spaces for params */
	    while (*params && (*params == ' '))
		params++;
	} else
	    cmd_len = strlen(line_read);
	
	entry = commands_list;
	while (entry && entry->cmd_name) {
	    if (!strncmp(entry->cmd_name, line_read, cmd_len)) {
		ret = entry->cmd_func(params);
		if (ret < 0)
		    printf("Command %s failed with error message \"%s\" (%d)\n",
			   entry->cmd_name, strerror(errno), ret);
		break;
	    }
	    entry++;
	}
    }

    cmd_exit(NULL);

    return 0;
}



char **
command_completion(const char *text, int start, int end)
{
    rl_attempted_completion_over = 1;
    return rl_completion_matches(text, command_name_generator);
}

char *
command_name_generator(const char *text, int state)
{
    static int list_index, len;
    char *name;

    if (!state) {
        list_index = 0;
        len = strlen(text);
    }

    while ((commands_list[list_index].cmd_name) && (name = commands_list[list_index++].cmd_name)) {
        if (strncmp(name, text, len) == 0) {
            return strdup(name);
        }
    }

    return NULL;
}
